function [F,g]=compute_length(Y,pars)

% Called by general_geodesics, computes 
% the length functional and its gradient.
%
% Please note: one may change the number of
% nodes used for integration (which is called K
% and is currently set to 10 times the number
% of Fourier parameters).

n=pars.n;
n2=n*n;
N=length(Y)/n2+1;
Xa=pars.Xa;
Xb=pars.Xb;

D=Xb-Xa;

X=[Xa;Xb;ones(n2,1);Y];
% compute function to be integrated
K=20*N; % number of nodes for integration
g=zeros(K,1);
t=1/(K-1);  % time interval
for j=1:K
    auxX=(X(2*n2+1:3*n2)).*(Xa+(j-1)*t*D);  % linear component
    Xdot=(X(2*n2+1:3*n2)).*D;
    T=(j-1)*t;
    for i=1:N-1
        auxX=auxX+X((i+2)*n2+1:(i+3)*n2)*sin(i*pi*T);
        Xdot=Xdot+i*pi*X((i+2)*n2+1:(i+3)*n2)*cos(i*pi*T);
    end
    [u,ss,v]=svd(reshape(auxX,n,n));
    s=diag(ss);
    f(j)=norm(Xdot)/s(n);
end
F=0;
% compute integral of f (naively for now)
for j=1:K-1
    g(j)=t*(f(j+1)+f(j))/2;
    F=F+g(j);
end