% Computes geodesics w.r.t. the condition metric
% using a Fourier-like parameterization and 
% M.L.Overton's BFGS implementation to minimize
% the length functional.
%
% P. Boito, December 2008
%

N=10; % number of parameters + 1
tolerance=1e-6;
K=10; % number of points where the curve is evaluated
H=10; % number of points for integration (when parameterizing
     % the condition number)

% set endpoints (matrices represented columnwise as vectors)

Xa=[2;0;0;1];   % length n2
Xb=[9;0;0;8];   % length n2

spacesize=sqrt(length(Xa));
if (spacesize==round(spacesize))&&(length(Xa)==length(Xb))
order=spacesize;
n2=order*order;
else
fprintf('Please check input.\n')
return
end

 if norm(Xa-Xb)<tolerance
     fprintf('Endpoints are too close.\n')
     return
 end  
 
 if abs(det(reshape(Xa,order,order)))<tolerance
     fprintf('Singular endpoint.\n')
     return
 end

 if abs(det(reshape(Xb,order,order)))<tolerance
     fprintf('Singular endpoint.\n')
     return
 end
 
 if (det(reshape(Xa,order,order))*det(reshape(Xb,order,order))<0)
     fprintf('No geodesic connecting these endpoints.\n')
     return
 end

% compute initial guess Y

X=zeros(n2*(N+2),1);
X(1:n2)=Xa;
X(n2+1:2*n2)=Xb;
X(2*n2+1:3*n2)=ones(n2,1);
Y=X(3*n2+1:end);

% minimize

pars.fgname='general_fungrad';
pars.Xa=Xa;
pars.Xb=Xb;
pars.n=order;
options.x0=Y;
options.maxit=100;
pars.nvar=(N-1)*n2;
[Z,ff,gg,hh]=bfgs(pars,options);
[F,g]=compute_length(Z,pars);
Z=[Xa;Xb;ones(n2,1);Z];

% recover geodesic
K=length(g);
milestones=zeros(K,1);
milestones(1)=0;
for k=1:K-1
    %milestones(k+1)=milestones(k)+lengths(k);
    milestones(k+1)=milestones(k)+g(k);
end
W=recover_curve(Z,n2,K-2);
 sv=zeros(K,1);   % values of the condition number
 for j=1:K
     S=reshape(W(:,j),order,order);
     s=svd(S);
     sv(j)=1/s(order);
 end
 figure(1)
plot(milestones,sv,'-mo')